//-------------------------------------------------------
//  ssol/draw.c
//			This is the main C file for our SVG file rendering 
//			utility as well as complex type constructors,
//      to be linked in with the LLVM code 
//			generated by the SSOL compiler.
//
//			This code is mostly not ours! The original
//			source code was taken from here:
//				http://www.code-in-c.com/writing-svg-library-c/
//
//			See also svg.h and svg.c
//      
//      Additional authorship: Daniel Mesko
//
//-------------------------------------------------------

#include<stdio.h>
#include<math.h>
#include<time.h>

#include "svg.h"

#define POINT 0

//--------------------------------------------------------
// FUNCTION PROTOTYPES
//--------------------------------------------------------
void read_canvas(struct canvas_node *node, svg *psvg);
int draw(struct canvas canv, char *filename);

// --------------------------------------------------------
// TYPE CONSTRUCTORS
// --------------------------------------------------------

struct point Point(double x, double y)
{
		struct point pt;
		pt.x = x;
		pt.y = y;
		return pt;
}

struct curve Curve(struct point ep1, struct point ep2, struct point cp1, struct point cp2)
{
		struct curve cv;
		cv.ep1 = ep1;
		cv.ep2 = ep2;
		cv.cp1 = cp1;
		cv.cp2 = cp2;
		return cv;
}

struct canvas Canvas(double x, double y)
{
		struct canvas c;
	
		c.x = x;
		c.y = y;
		c.first = 0;
		return c;
}

// ------------------------------------------------------
//  CANVAS READING/SVG RENDERING FUNCTIONS
// ------------------------------------------------------

int draw(struct canvas canv, char *filename)
{
		svg* psvg;
		psvg = svg_create(canv.x, canv.y);

		if (psvg == NULL) {
				fprintf(stderr, "could not store SVG meta data, malloc returned null");
				exit(1);
		}
		else{
			read_canvas(canv.first, psvg);
			
			svg_finalize(psvg);
			svg_save(psvg, filename);
			svg_free(psvg);
		}

		return 0;
}
void read_canvas(struct canvas_node *node, svg *psvg)
{	
		// Walk the canvas node list, render each curve element

		if (node == NULL)
				return;
		
		struct canvas_node *next = node->next;
		struct curve *ct = node->ct;

		int ep1x = (int) ct->ep1.x;
		int ep1y = (int) ct->ep1.y;
		int ep2x = (int) ct->ep2.x;
		int ep2y = (int) ct->ep2.y;
		int cp1x = (int) ct->cp1.x;
		int cp1y = (int) ct->cp1.y;
		int cp2x = (int) ct->cp2.x;
		int cp2y = (int) ct->cp2.y;

		svg_bezier(psvg, ep1x, ep1y, ep2x, ep2y, cp1x, cp1y, cp2x, cp2y); 
		
		read_canvas(next, psvg);

}

